@@ -293,6 +293,7 @@ class BigQueryExecutor:
                        project_id=bq_default_project(),
                        partition=False,
                        partition_field='_PARTITIONTIME',
+                       require_partition_filter=False,
                        clustering=None):
         """ Initiate a BigQuery table. If the table already exists, compares the schema_path and apply a patch if there is a schema change.
 
@@ -303,7 +304,11 @@ class BigQueryExecutor:
             schema_path (string: Path to the BigQuery table schema from the PROJECT_ROOT environement variable.
             partition (bool): Specify whether the BigQuery table is partioned. Default to False.
             partition_field (string): Specify partition_field if table partioned. Defaults to "_PARTITIONTIME".
+            require_partition_filter (bool): Specify whether querying the table would require a partition filter. Defaults to False.
             clustering (list): List of clustering fields. Defaults to None.
+        
+        Note:
+            partitioning and clustering require reinitialisation of the table (existing table won't be patched)
         """
         if self.table_exists(
                 project_id=project_id,
@@ -334,6 +339,7 @@ class BigQueryExecutor:
                         )
                     else:
                         raise ValueError("partition_field should be a string")
+                table.require_partition_filter = require_partition_filter
             table.clustering_fields = clustering
             try:
                 table = self.client.create_table(table)
@@ -351,6 +357,7 @@ class BigQueryExecutor:
                      location=bq_default_location(), schema_path='',
                      partition=False,
                      partition_field='_PARTITIONTIME',
+                     require_partition_filter=False,
                      clustering=None,
                      priority='INTERACTIVE',
                      description=None,
@@ -368,6 +375,7 @@ class BigQueryExecutor:
             schema_path:
             partition:
             partition_field:
+            require_partition_filter (bool): Specify whether querying the table would require a partition filter. Defaults to False.
             clustering: A list of field names that the table will be clustered on, e.g. [field1, field2]
             priority:
             description:
@@ -390,6 +398,7 @@ class BigQueryExecutor:
                 schema_path=schema_path,
                 partition=partition,
                 partition_field=partition_field,
+                require_partition_filter=require_partition_filter,
                 dataset_id=dataset_id,
                 project_id=project_id,
                 clustering=clustering
@@ -501,8 +510,6 @@ class BigQueryExecutor:
         Returns:
             None
         """
-        client = bigquery.Client()
-
         sql = f"""
                DELETE
                  `{dataset_id}.{table_id}`
@@ -511,12 +518,10 @@ class BigQueryExecutor:
                    CAST('{from_date}' AS DATE) AND CAST('{to_date}' AS DATE)
                """
 
-        query_job = client.query(
-            sql
-        )
-
+        query_job = self.client.query(sql)
         query_job.result() # wait for job to complete
 
+
     def delete_backfill_partitions(self,
                                    backfill_dataset_id,
                                    table_id,
@@ -537,8 +542,6 @@ class BigQueryExecutor:
         Returns:
             None
         """
-        client = bigquery.Client()
-
         sql = f"""
                DELETE
                  `{backfill_dataset_id}.{table_id}`
@@ -547,10 +550,7 @@ class BigQueryExecutor:
                    CAST('{from_partition}' AS DATE) AND CAST('{to_partition}' AS DATE)
                """
 
-        query_job = client.query(
-            sql
-        )
-
+        query_job = self.client.query(sql)
         query_job.result() # wait for job to complete
 
 
@@ -560,7 +560,9 @@ class BigQueryExecutor:
                           table_id,
                           schema_path,
                           sql,
-                          partition_field
+                          partition_field,
+                          require_partition_filter=False,
+                          clustering=None
                           ):
         """
         Run a full backfill for a time-unit column partitioned table
@@ -574,6 +576,8 @@ class BigQueryExecutor:
                 Note: it has to contain the clause:
                 WHERE <partition_field> BETWEEN CAST(<from_partition> AS DATE) AND CAST(<to_partition> AS DATE)
             partition_field (string): the time-unit column that the table is partitioned by
+            require_partition_filter (bool): Specify whether querying the table would require a partition filter. Defaults to False.
+            clustering (list): List of clustering fields. Defaults to None.
 
         Returns:
             None
@@ -593,6 +597,8 @@ class BigQueryExecutor:
             sql=sql,
             partition=True,
             partition_field=partition_field,
+            require_partition_filter=require_partition_filter,
+            clustering=clustering,
             write_disposition="WRITE_TRUNCATE"
         )
 
@@ -620,7 +626,9 @@ class BigQueryExecutor:
                              sql,
                              partition_field,
                              from_partition,
-                             to_partition
+                             to_partition,
+                             require_partition_filter=False,
+                             clustering=None
                              ):
         """
         Run a partial backfill for a time-unit column partitioned table between the dates
@@ -637,6 +645,8 @@ class BigQueryExecutor:
             partition_field (string): the time-unit column that the table is partitioned by
             from_partition (date): date of the first partition to be backfilled
             to_partition (date): date of the latest partition to be backfilled
+            require_partition_filter (bool): Specify whether querying the table would require a partition filter. Defaults to False.
+            clustering (list): List of clustering fields. Defaults to None.
 
         Returns:
             None
@@ -674,6 +684,8 @@ class BigQueryExecutor:
             sql=sql,
             partition=True,
             partition_field=partition_field,
+            require_partition_filter=require_partition_filter,
+            clustering=clustering,
             write_disposition="WRITE_APPEND"
         )
 
@@ -701,7 +713,9 @@ class BigQueryExecutor:
                      sql,
                      partition_field,
                      from_partition,
-                     to_partition
+                     to_partition,
+                     require_partition_filter=False,
+                     clustering=None
                      ):
         """
         Run a full or partial backfill for a time-unit column partitioned table
@@ -715,6 +729,8 @@ class BigQueryExecutor:
                 Note: it has to contain the clause:
                 WHERE <partition_field> BETWEEN CAST(<from_partition> AS DATE) AND CAST(<to_partition> AS DATE)
             partition_field (string): the time-unit column that the table is partitioned by
+            require_partition_filter (bool): Specify whether querying the table would require a partition filter. Defaults to False.
+            clustering (list): List of clustering fields. Defaults to None.
 
         Returns:
             None
@@ -735,7 +751,9 @@ class BigQueryExecutor:
                 table_id=table_id,
                 schema_path=schema_path,
                 sql=sql,
-                partition_field=partition_field
+                partition_field=partition_field,
+                require_partition_filter=require_partition_filter,
+                clustering=clustering
             )
 
         elif backfill == "partial":
@@ -747,9 +765,12 @@ class BigQueryExecutor:
                 sql=sql,
                 partition_field=partition_field,
                 from_partition=from_partition,
-                to_partition=to_partition
+                to_partition=to_partition,
+                require_partition_filter=require_partition_filter,
+                clustering=clustering
             )
 
+
     def create_partitions_legacy(self,
                                  sql,
                                  file,
@@ -825,6 +846,7 @@ class BigQueryExecutor:
                                partition_dates=None,
                                pipeline_partition_date=None,
                                partition_field='_PARTITIONTIME',
+                               require_partition_filter=False,
                                clustering=None,
                                priority="INTERACTIVE",
                                schema_path="",
@@ -854,6 +876,7 @@ class BigQueryExecutor:
                 schema_path=schema_path,
                 partition=True,
                 partition_field=partition_field,
+                require_partition_filter=require_partition_filter,
                 clustering=clustering
             )
 
@@ -904,6 +927,7 @@ class BigQueryExecutor:
                     sql=sql,
                     partition=True,
                     partition_field=partition_field,
+                    require_partition_filter=require_partition_filter,
                     write_disposition=write_disposition,
                     use_legacy_sql=use_legacy_sql,
                     clustering=clustering,
@@ -927,7 +951,9 @@ class BigQueryExecutor:
                 sql=sql,
                 partition_field=partition_field,
                 from_partition=from_partition,
-                to_partition=to_partition
+                to_partition=to_partition,
+                require_partition_filter=require_partition_filter,
+                clustering=clustering
             )
 
         if description:
@@ -1577,7 +1603,7 @@ class BigQueryExecutor:
         else:
             raise Exception("Please initiate %s:%s.%s or pass the schema file",project_id,dataset_id, table_id)
 
-    def load_gcs(self, dataset_id, table_id, gcs_path, gcs_bucket=gcs_default_bucket(),project_id=bq_default_project(), location='US', schema_path='', header=True, write_disposition='WRITE_TRUNCATE'):
+    def load_gcs(self, dataset_id, table_id, gcs_path, gcs_bucket=gcs_default_bucket(),project_id=bq_default_project(), location='US', schema_path='', header=True, write_disposition='WRITE_TRUNCATE', allow_quoted_newlines=True):
         """ Loads Google Cloud Storage CSV file into a BigQuery table.
 
         Parameters:
@@ -1617,6 +1643,7 @@ class BigQueryExecutor:
 
         job_config.source_format = bigquery.SourceFormat.CSV
         job_config.write_disposition = write_disposition
+        job_config.allow_quoted_newlines = allow_quoted_newlines
 
         table_ref = self.get_table_ref(
             dataset_id=dataset_id,
